#include <QBoxLayout>
#include <QPixmap>
#include <QIcon>
#include <QLabel>
#include <QTextEdit>
#include <QStyleOption>
#include <QPainter>
#include <QTextDocument>
#include <QDesktopServices>
#include <QGuiApplication>
#include "kaboutdialog.h"

namespace kdk
{
class KAboutDialogPrivate:public QObject
{
    Q_OBJECT
    Q_DECLARE_PUBLIC(KAboutDialog)

public:
    KAboutDialogPrivate(KAboutDialog*parent);
    void adjustMiniMode();
    void adjustNormalMode();

private:
    KAboutDialog *q_ptr;
    QLabel *m_pAppIconLabel;
    QLabel *m_pAppNameLabel;
    QLabel *m_pAppVersionLabel;
    QLabel *m_pBodyTextEdit;
    QLabel *m_pAppSupportLabel;
    QString m_iconName;
    bool m_pIsVisable;
};

KAboutDialog::KAboutDialog(QWidget*parent,const QIcon& appIcon,const QString& appName,const QString& appVersion,const QString& appInfo)
    :KDialog(parent),
      d_ptr(new KAboutDialogPrivate(this))
{
    Q_D(KAboutDialog);
    mainLayout()->setSizeConstraint(QLayout::SizeConstraint::SetFixedSize);
    d->m_iconName = appIcon.name();
    d->m_pAppIconLabel = new QLabel(this);
    QPixmap pix(appIcon.pixmap(QSize(96,96)));
    d->m_pAppIconLabel->setPixmap(pix);
    d->m_pAppIconLabel->setAlignment(Qt::AlignCenter);
    d->m_pAppNameLabel = new QLabel(this);
    d->m_pAppNameLabel->setText(appName);
    d->m_pAppNameLabel->setAlignment(Qt::AlignCenter);

    d->m_pAppVersionLabel = new QLabel(this);
    d->m_pAppVersionLabel->setText(appVersion);
    d->m_pAppVersionLabel->setAlignment(Qt::AlignCenter);

    d->m_pBodyTextEdit = new QLabel(this);
    d->m_pBodyTextEdit->setFixedWidth(370);
    d->m_pBodyTextEdit->setWordWrap(true);
    d->m_pBodyTextEdit->setScaledContents(true);
    d->m_pBodyTextEdit->setText(appInfo);
    d->m_pBodyTextEdit->setTextInteractionFlags(Qt::NoTextInteraction);

    d->m_pAppSupportLabel = new QLabel(this);
    d->m_pAppSupportLabel->setFixedWidth(370);

    if(appInfo.isEmpty())
    {
        d->m_pBodyTextEdit->setVisible(false);
        d->adjustMiniMode();
    }
    else
    {
        d->m_pBodyTextEdit->setVisible(true);
        d->adjustNormalMode();
    }
    connect(d->m_pAppSupportLabel,&QLabel::linkActivated,this,[=](const QString url){
        QDesktopServices::openUrl(QUrl(url));
    });
    changeTheme();
    connect(m_gsetting,&QGSettings::changed,this,[=](){changeTheme();});
}

KAboutDialog::~KAboutDialog()
{

}

void KAboutDialog::setAppIcon(const QIcon& icon)
{
    Q_D(KAboutDialog);
    d->m_iconName = icon.name();
    QPixmap pix(icon.pixmap(QSize(96,96)));
    d->m_pAppIconLabel->setPixmap(pix);
    d->m_pAppIconLabel->setAlignment(Qt::AlignCenter);
    update();
}


void KAboutDialog::setAppName(const QString& appName)
{
    Q_D(KAboutDialog);
    d->m_pAppNameLabel->setText(appName);
    update();
}

QString KAboutDialog::appName()
{
    Q_D(KAboutDialog);
    return d->m_pAppNameLabel->text();
}

void KAboutDialog::setAppVersion(const QString& appVersion)
{
    Q_D(KAboutDialog);
    d->m_pAppVersionLabel->setText(appVersion);
    update();
}

QString KAboutDialog::appVersion()
{
    Q_D(KAboutDialog);
    return d->m_pAppVersionLabel->text();
}

void KAboutDialog::setBodyText(const QString& bodyText)
{
    Q_D(KAboutDialog);
    d->m_pBodyTextEdit->setText(bodyText);
    update();
}

QString KAboutDialog::bodyText()
{
    Q_D(KAboutDialog);
    return d->m_pBodyTextEdit->text();
}

void KAboutDialog::setAppSupport(const QString& appSupport)
{
    Q_D(KAboutDialog);
    d->m_pAppSupportLabel->setText(appSupport);
    update();
}

QString KAboutDialog::appSupport()
{
    Q_D(KAboutDialog);
    return d->m_pAppSupportLabel->text();
}

void KAboutDialog::setBodyTextVisiable(bool flag)
{
    Q_D(KAboutDialog);
    d->m_pBodyTextEdit->setVisible(flag);
    d->m_pIsVisable = flag;

    update();
}

void KAboutDialog::changeTheme()
{
    Q_D(KAboutDialog);
    KDialog::changeTheme();

    auto icon = QIcon::fromTheme(d->m_iconName);
    if(!icon.isNull())
    {
        QPixmap pix(icon.pixmap(QSize(96,96)));
        d->m_pAppIconLabel->setPixmap(pix);
    }
    QFont font;
    font.setPixelSize(ThemeController::systemFontSize() * 1.7);
    font.setWeight(QFont::Medium);
    d->m_pAppNameLabel->setFont(font);
    d->m_pBodyTextEdit->setFrameStyle(0);
    QPalette palette = qApp->palette();
    if(ThemeController::themeMode() == LightTheme)
    {
       palette.setColor(QPalette::Text,QColor("#595959"));
       palette.setColor(QPalette::WindowText,QColor("#595959"));
       palette.setColor(QPalette::Base,QColor(0,0,0,0));
       d->m_pAppSupportLabel->setPalette(palette);
       d->m_pAppVersionLabel->setPalette(palette);
       d->m_pBodyTextEdit->setPalette(palette);
       d->m_pAppSupportLabel->setText(tr("Service & Support: ") +
                                              "<a href=\"mailto://support@kylinos.cn\""
                                              "style=\"color:#595959\">"
                                              "support@kylinos.cn</a>");
    }
    else
    {
        palette.setColor(QPalette::ButtonText,QColor("#D9D9D9"));
        palette.setColor(QPalette::WindowText,QColor("#D9D9D9"));
        d->m_pAppSupportLabel->setPalette(palette);
        d->m_pAppVersionLabel->setPalette(palette);
        d->m_pBodyTextEdit->setPalette(palette);
        d->m_pAppSupportLabel->setText(tr("Service & Support: ") +
                                             "<a href=\"mailto://support@kylinos.cn\""
                                             "style=\"color:#D9D9D9\">"
                                             "support@kylinos.cn</a>");
    }
}

void KAboutDialog::paintEvent(QPaintEvent *paintEvent)
{
    Q_D(KAboutDialog);

    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
    if(ThemeController::widgetTheme() == FashionTheme)
    {
        auto pixmap = ThemeController::drawFashionBackground(opt.rect,32,36,8,0);
        p.drawPixmap(opt.rect,pixmap);
    }
    if(d->m_pIsVisable && !d->m_pBodyTextEdit->text().isEmpty())
    {
        d->adjustNormalMode();
    }
    else
    {
        d->adjustMiniMode();
    }
}

KAboutDialogPrivate::KAboutDialogPrivate(KAboutDialog *parent)
    :q_ptr(parent)
{
    setParent(parent);
}

void KAboutDialogPrivate::adjustMiniMode()
{
    Q_Q(KAboutDialog);
    if(q->mainWidget()->layout())
        delete q->mainWidget()->layout();
    m_pAppSupportLabel->setAlignment(Qt::AlignCenter);
    QVBoxLayout *vBoxLayout = new QVBoxLayout();
    vBoxLayout->setSpacing(0);
    vBoxLayout->addSpacing(24);
    vBoxLayout->addWidget(m_pAppIconLabel);
    vBoxLayout->addSpacing(16);
    vBoxLayout->addWidget(m_pAppNameLabel);
    vBoxLayout->addSpacing(12);
    vBoxLayout->addWidget(m_pAppVersionLabel);
    vBoxLayout->addSpacing(12);
    vBoxLayout->addWidget(m_pAppSupportLabel);
    vBoxLayout->setContentsMargins(25,0,25,0);
    vBoxLayout->addSpacing(40);
    q->mainWidget()->setLayout(vBoxLayout);
}

void KAboutDialogPrivate::adjustNormalMode()
{
    Q_Q(KAboutDialog);
    if(q->mainWidget()->layout())
        delete q->mainWidget()->layout();
    m_pAppSupportLabel->setAlignment(Qt::AlignLeft);
    QVBoxLayout *vBoxLayout = new QVBoxLayout();
    vBoxLayout->setSpacing(0);
    vBoxLayout->addSpacing(20);
    vBoxLayout->addWidget(m_pAppIconLabel);
    vBoxLayout->addSpacing(16);
    vBoxLayout->addWidget(m_pAppNameLabel);
    vBoxLayout->addSpacing(12);
    vBoxLayout->addWidget(m_pAppVersionLabel);
    vBoxLayout->addSpacing(12);
    vBoxLayout->addWidget(m_pBodyTextEdit);
    vBoxLayout->addSpacing(24);
    vBoxLayout->addWidget(m_pAppSupportLabel);
    vBoxLayout->setContentsMargins(25,0,25,0);
    vBoxLayout->addSpacing(40);
    q->mainWidget()->setLayout(vBoxLayout);
}

}

#include "kaboutdialog.moc"
#include "moc_kaboutdialog.cpp"
